"use strict";
const path        = require('path')
const isUp        = require('is-up')
const isThere     = require('is-there')
const isOnline    = require('is-online')
const sortObj     = require('sort-object')
const configstore = require('configstore')
const git         = require('git-state')
const gd          = require('../../helper/gd.js')
const timeago     = require('../../helper/timeago')
const support     = require('../../helper/support.js')
const gitlab      = require('../../gitlab/gitlab.js')

module.exports = function(gd_cli_path, theme_path, cb) {
  isOnline().then(online => {
    menu(gd_cli_path, theme_path, cb, online)
  })
}


const menu = function(gd_cli_path, theme_path, cb, online) {

  const pkg  = theme_path + '/package.json'
  const conf = isThere(pkg) ? require(pkg) : false;

  if(!conf) {
    console.log(`⁉️\n---\nUnable to find ${pkg} | color=#ff9a7b`);return;
  }

  gd.package = conf // IMPORTANT, Manually set current conf

  const isDarkMode = process.env.BitBarDarkMode === '1';

  const conf_local = gd.localConf();

  /* INIT GD-GUI CONF */
  const conf_gd_cli = new configstore('gd-cli-' + conf.name);

  const C = conf;
  const S = conf.servers;

  C.theme = theme_path;
  const type    = C.config.type
  const version = C.config.version
  const nosql   = type === 'static' || type === 'react-native' || type === 'node'

  conf.site_local = support.findRoot(C, C.theme)
  conf.site_root  = conf.site_local.replace(/\/web|\/www|\/app/i, '/')

  let cc_cmd = null;

  if(!support.isCommandExcluded('clear-cache')){
    cc_cmd = 'Clear cache';
    if(type === 'drupal' && conf.config.version !== '7.x') {
      cc_cmd = 'Cache rebuild';
    }
  }

  const cd_theme = 'cd ' + C.theme + ' && ';
  const cd_site  = 'cd ' + C.site_local + ' && ';

  const fontSize      = '17';
  const fontSizeSmall = '14';
  const fontSizeTitle = '14';
  const fontTitle     = 'Trebuchet MS Bold';
  const colorDown     = isDarkMode ? '#9bd28e' : '#156f00';
  const colorUp       = isDarkMode ? '#ff9a7b' : '#b44000';
  const colorDis      = isDarkMode ? '#bbbbbb' : '#888888';
  const colortitle    = isDarkMode ? '#fff261' : '#493e21';

  git.check(conf.site_root, function (err, result) {
    let untracked = ''
    let dirty     = ''
    let gitstatus = ''
    //if (err) throw err
    //console.log(result) // => { branch: 'master', 
                          //      ahead: 0, 
                          //      dirty: 9, 
                          //      untracked: 1, 
                          //      stashes: 0 } 
    if(result.dirty > 0){
      dirty = 'Changed: ' + result.dirty
    }
    if(result.untracked > 0){
      if(dirty){
        dirty+= ', '
      }
      untracked = 'Untracked: ' + result.untracked
    }

    gitstatus = untracked || dirty ? '[git] ' + dirty + untracked : '';

    conf_gd_cli.set('gitstate', gitstatus);
  })



  // BITBAR TITLE
  const shortname_title = C.shortname ? C.shortname.toUpperCase() : '';
  if(C['ico'] && /\.png|\.gif|\.jpg/.test(C['ico'])) {

    const image = new Buffer(require('fs').readFileSync(C.theme + '/' + C['ico'])).toString('base64');

    console.log(`| image="${image}"`);
  }
  else if(C['ico']) {
    console.log(`| image="${C['ico']}"`);
  }
  else {
    console.log(shortname_title || C.name);
  }

  sep();

  let gitstatus = conf_gd_cli.get('gitstate');
  if(gitstatus){
    console.log(`${gitstatus} | color=${colorDis} size=13 bash='${cd_site}git status'`)
    sep();
  }

  // Async Check if site is up
  for (let server in conf.servers) {

    if(server === 'dev') continue;

    if(!online || !conf.servers[server].url) {
      conf_gd_cli.set('isup-' + server, 'none');
    }
    else {
      ((s ,url) => {
        try{
          isUp(url).then(up => {
            conf_gd_cli.set('isup-' + s, up);
          });
        } catch(err) { }
      })(server, conf.servers[server].url)
    }
  }



  const menuContent = function(project, online) {
    
    const exit  = ' && exit';
    const gdcmd = 'gd-gui ';

    const username = gd.username();


    if(project){
      const issues          = project.issues;
      const nb_issues       = project.infos.open_issues_count;
      const nb_issues_label = nb_issues > 1 ? 'issues' : 'issue';
      const last_activity   = timeago().ago(project.infos.last_activity_at);
      const project_path    = project.infos.path_with_namespace;
      const web_url         = `http://${gitlab.domain()}/${project_path}/`;

      const issue_ico = nb_issues > 0 ? '🚧' : '';
      const issue_col = nb_issues > 0 ? colorUp : colorDis;
      const issue_fs  = nb_issues > 0 ? fontSize : fontSizeSmall;

      console.log(`${issue_ico} ${nb_issues} ${nb_issues_label} | color=${issue_col} href='${web_url}issues' size=${issue_fs}`);


      function printIssue(issue, params) {
        let i = issues[issue]
        let title = i.title.substr(0,68);
        let issue_url = `${web_url}/issues/${i.iid}'`;
        let assignee = '';

        if(!params && i['assignee'] && i['assignee']['username']) {
          assignee = '⤏ ' + i.assignee.username
        }
        sub(`${title} ${assignee} | href='${issue_url}' ${params}`);
      }

      for(var issue in issues) {
        if(issues[issue]['assignee'] && issues[issue]['assignee']['username'] === username) {
          printIssue(issue, `color=${colorDown}`);
        }
      }
      for(var issue in issues) {
        if(!issues[issue]['assignee'] || issues[issue]['assignee']['username'] !== username) {
          printIssue(issue);
        }
      }


      alt(`➠ Create an issue | color=${colorDown} href='${web_url}issues/new' size=${issue_fs}`);

      console.log(`⚡️ Activity ⤏ ${last_activity} | color=${colorDown} href='${web_url}activity' size=${fontSizeSmall}`);
      console.log(`📄 Readme.md | alternate=true color=${colorDown} href='${web_url}blob/master/README.md' size=${fontSizeSmall}`);
    }

    sep();
    
    // DEV //
    let dev_url = gd.dev_url();

    t(`DEV | href='http://${dev_url}'`)

    // DEV INFOS
    console.log('-----');
    sub(`Dump Database | bash='${cd_theme}${gdcmd}dump -s dev' color=${colorDown}`);
    subtitle(`INFOS : |`);
    sub(`URL : ${dev_url} | bash='echo ${dev_url} | pbcopy${exit}'`)

    var db_name = gd.db('dev');
    sub(`DB NAME : ${db_name} | bash='echo ${db_name} | pbcopy${exit}'`)
    sub(`CREATE DB | bash='${cd_theme}${gdcmd}create-db -s dev${exit}' color=${colorDown} alternate=true`);

    if(!support.isCommandExcluded('settings')
    && !isThere(C.site_local + '/sites/default/settings.php')){
      sub(`Create settings.php | bash='${cd_theme}${gdcmd}settings -s dev${exit}' color=${colorDown}`)
    }

    talt(`DEV ⤏ Open in ST | bash='subl ${C.site_local}${exit}'`)


    let gdPull = `${gdcmd}pull -s dev`;
    if(cc_cmd) {
      gdPull = `gd pull -s dev && ${gdcmd}cc -s dev`
    }

    l(`⇣ Pull | color=${colorDown} bash='${cd_site}${gdPull}${exit}'`)
    alt(`⇡ Push | color=${colorUp} bash='${cd_site}git add . && git commit -am . && git pull && git push && sleep 1${exit}'`)

    if(isThere(`${theme_path}/gulpfile.js`)) {
      l(`➠ Gulp | color=${colorDown} bash='${cd_theme}${gdcmd}gulp'`)
    }

    if(cc_cmd) {
      alt(`⟲ ${cc_cmd} | color=${colorDown} bash='${cd_theme}${gdcmd}cc -s dev${exit}'`)
    }

    if(!support.isCommandExcluded('composer') || isThere(conf.site_local+'/composer.json')){
      l(`✛ Composer install | color=${colorDown} bash='${cd_theme}${gdcmd}composer -s dev'`)
      alt(`⟲ Composer update | color=${colorDown} bash='${cd_theme}${gdcmd}composer update -s dev'`)
    }

    sep();

    // SERVERS //
    for (let server in conf.servers) {
      if(server === 'dev') continue;

      let server_conf = conf.servers[server];

      let up_ico = '';
      let up = conf_gd_cli.get('isup-' + server);
      if(up !== undefined) {
        const up_code = up == null ? '' : `(${up})`;
        up_ico = up === true ? '✓' : up === 'none' ? '' : `❗️ ${up_code}`; /*'⊘'*/
      }

      let server_url = conf.servers[server].url;

      if(server_conf['url_auth']){
        server_url = server_conf.url_auth + '@' + server_url;
      }

      t(`${server.toUpperCase()} ${up_ico} | href='http://${server_conf.url}'`);
      sub(`⬆︎ Deploy | color=${colorUp} bash='${cd_theme}${gdcmd}deploy -s ${server}${exit}'`)
      sub(`⇧ Composer install | color=${colorUp} bash='${cd_theme}${gdcmd}composer -s ${server}${exit}'`)

      if(cc_cmd) {
        sub(`⟲ ${cc_cmd} | color=${colorDown} bash='${cd_theme}${gdcmd}cc -s ${server}${exit}'`)
      }
      

      console.log('-----');
      subtitle(`INFOS : |`);
      sub(`SSH : ${server_conf.user}@${server_conf.ip}:${server_conf.port} | color=${colorDis} bash='ssh -p ${server_conf.port} ${server_conf.user}@${server_conf.ip}'`)
      sub(`URL : ${server_conf.url} | color=${colorDis} href='http://${server_url}'`)

      if(server_conf['url_auth']){
        sub(`auth : ${server_conf['url_auth']} | color=${colorDis} href='http://${server_url}'`)
      }
      
      talt(`${server.toUpperCase()} ⤏ SSH | bash='ssh -p ${server_conf.port} ${server_conf.user}@${server_conf.ip}'`);

      l(`⇧ Update | color=${colorUp} bash='${cd_theme}${gdcmd}update -s ${server}${exit}'`);
      alt(`⇧ Simple Update | color=${colorUp} bash='${cd_theme}${gdcmd}simple-update -s ${server}${exit}'`);
      // alt(`⇧ Push & Update | color=${colorUp} bash='${cd_site}git add . && git commit -am . && git pull && git push && ${cd_theme}${gdcmd}update -s ${server}${exit}'`);

      if(!nosql) {
        l(`⟲ Import DB | color=${colorDown} bash='${cd_theme}${gdcmd}get-db -s ${server}${exit}'`);
        alt(`⬇︎ Download files | color=${colorDown} bash='${cd_theme}${gdcmd}get-files -s ${server}${exit}'`);
      }
      else {
        l(`⬇︎ Download files | color=${colorDown} bash='${cd_theme}${gdcmd}get-files -s ${server}${exit}'`);
      }
      //echo "🌀 ${cc_cmd} | bash='$THEME gd clear-cache -s ${server}' alternate=true size=${fontSize}"

      sep();
    }

    /* CUSTOM LINK */
    if(C['links']) {
      sep();
      for (let link in C['links']){
        l(`${link} | href='${C['links'][link]}'`);
      }
      sep();
    }


    /* PLUGINS JS */
    if(!support.isCommandExcluded('plugin')) {
      l(`✜ Plugins JS | color=${colorDown} bash='${cd_theme}${gdcmd}plugin${exit}'`);

      let plugins = require('../../config/plugin.json');

      // hide drupal.js plugin from drupal project
      if(type === 'drupal'){
        delete plugins['drupal.js'];
      }

      plugins = sortObj(plugins);

      for (var name in plugins) {

        if(plugins[name]['disabled']) continue;

        if(!C.plugins || C.plugins && (!C.plugins[0] || C.plugins.indexOf(name) == -1)) {
          sub(`${name} | bash='${cd_theme}${gdcmd}plugin ${name}${exit}'`);

          if(plugins[name]['url']){
            for (var u in plugins[name]['url']){
              subsub(`${u} | href='${plugins[name]['url'][u]}'`)
            }
          }
        }
      }
      var i = 0;
      for (var name in plugins) {

        if(C.plugins && C.plugins[0] && C.plugins.indexOf(name) > -1) {

          if(i === 0){
            console.log('-----');
            subtitle(`INSTALLED : |`);
          }
          sub(`${name} | bash='${cd_theme}${gdcmd}plugin ${name}${exit}'`);
          
          if(plugins[name]['url']){
            for (var u in plugins[name]['url']){
              subsub(`${u} | href='${plugins[name]['url'][u]}'`)
            }
          }

          sub(`✘ ${name} | bash='${cd_theme}${gdcmd}plugin-remove ${name}${exit}' color=${colorUp} alternate=true`);

          i++;
        }
      }
      console.log('-----');
      sub(`⟲ Update | color=${colorDown} bash='${cd_theme}ncu -m bower -ua && bower update'`)

    }

    sep();


    const composers = sortObj(require('../../config/composer.json'));

    let required = false;

    if(isThere(conf.site_local + '/composer.json')) {
      required = sortObj(require(conf.site_local + '/composer.json'));
      required = required['require'];
    }
    else if(isThere(conf.site_local.replace('/web', '') + '/composer.json')) {
      required = sortObj(require(conf.site_local.replace('/web', '') + '/composer.json'));
      required = required['require'];
    }

    l(`✜ Composer package | color=${colorDown}`);
    for (var P in composers) {

      if(!required || required && !required[P]){
        sub(`${P} | bash='${cd_theme}${gdcmd}composer ${P} -s dev${exit}'`);
          
        if(composers[P]['url']){
          for (var u in composers[P]['url']){
            subsub(`${u} | href='${composers[P]['url'][u]}'`)
          }
        }
        subsub(`Packagist | href='https://packagist.org/packages/${P}'`)
      }
    }
    if(required){

      console.log('-----');
      subtitle(`INSTALLED : |`);

      for (var R in required) {

        sub(`${R} | bash='${cd_theme}${gdcmd}composer ${R} -s dev${exit}'`);
          
        if(composers[R] && composers[R]['url']){
          for (var u in composers[R]['url']){
            subsub(`${u} | href='${composers[R]['url'][u]}'`)
          }
        }
        subsub(`Packagist | href='https://packagist.org/packages/${R}'`)
      }
      console.log('-----');
      sub(`⟲ Update packages | color=${colorDown} bash='${cd_theme}${gdcmd}composer update -s dev'`)

    }

    sep();








    /* PLUGINS WORDPRESS */
    if(!support.isCommandExcluded('wordpress')) {
      l(`✜ Plugins WP | color=${colorDown}`);

      var plugins = require('../../config/plugin-wordpress.json');

      plugins = sortObj(plugins);

      for (var name in plugins) {
        sub(`${name} | bash='${cd_theme}${gdcmd}wp-plugin ${plugins[name].name} -s dev${exit}'`);

        if(plugins[name]['url']){
          for (var u in plugins[name].url){
            subsub(`${u} | href='${plugins[name]['url'][u]}'`)
          }
        }
      }
      console.log('-----');
      sub(`⟲ Update plugins | color=${colorDown} bash='${cd_theme}${gdcmd}wp-plugin update -s dev${exit}'`)
      sub(`⟲ Update core | color=${colorDown} bash='${cd_theme}${gdcmd}wp update -s dev${exit}'`)
    }

    sep();







    console.log(`✗ Remove menu | color=${colorUp} bash='${cd_theme}${gdcmd}gui-remove${exit}' size=${fontSizeSmall}`);
    console.log(`⟲ Reload menu | alternate=true color=${colorDown} bash='${cd_theme}gd gui-remove && gd gui${exit}' size=${fontSizeSmall}`);

    if(cb) cb();
  }





  // MENU CONTENT

  if(online){
    gitlab.project.get(function(project){
      menuContent(project, online)
    }) 
  }
  else{
    menuContent(null, online)
  }

  







  function l(l) {
    console.log(`${l} size=${fontSize}`);
  }
  function sub(l) {
    console.log(`--${l} size=${fontSizeSmall}`);
  }
  function subtitle(l) {
    console.log(`--${l} font='${fontTitle}' color=${colortitle}`);
  }
  function subsub(l) {
    console.log(`----${l} size=${fontSizeSmall}`);
  }
  function t(l) {
    console.log(`${l} font='${fontTitle}' color=${colortitle} size=${fontSizeTitle}`);
  }
  function talt(l) {
    console.log(`${l} font='${fontTitle}' color=${colortitle} size=${fontSizeTitle} alternate=true`);
  }
  function alt(l) {
    console.log(`${l} alternate=true size=${fontSize}`);
  }
  function sep() {
    console.log('---');
  }

};



