<?php

global $wp_form_state;

/**
 * wp_get_form
 * @param $form_id
 * @param array $args
 * @param bool $render_form
 * @return array|string
 */
function wp_get_form($form_id, $args = array(), $render_form = true) {

  global $wp_form_state;

  if(!function_exists($form_id))
    return ($render_form) ? '' : array();

  // Default form state
  $form_state = array(
    'method' => 'POST',
    'redirect' => '//' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'],
    'values' => array(),
    'errors' => array()
  );

  $form['form_id'] = $form_id;
  $form['form_state'] = $form_state;

  // Set submitted values
  if(!empty($wp_form_state['values']['wp_form_id']) && $wp_form_state['values']['wp_form_id'] == $form_id) {
    $form['form_state']['values'] = $wp_form_state['values'];
    $form['form_state']['errors'] = (!empty($wp_form_state['errors'])) ? $wp_form_state['errors'] : array();
  }

  // Get form fields
  $form['form_items'] = $form_id(array(), $form_state, $args);

  // add wp nonce field
  $form['form_items'][$form_id . '_nonce'] = array(
    '#type' => 'nonce',
    '#referer' => true,
    '#echo' => false,
  );

  // add wp form id field
  $form['form_items']['wp_form_id'] = array(
    '#type' => 'hidden',
    '#default_value' => $form_id
  );

  return ($render_form) ? wp_render_form($form) : $form;
}

/**
 * wp_render_form
 * @param $form
 * @return string
 */
function wp_render_form($form) {

  $form_attr_id = str_replace('_', '-', $form['form_id']);
  $form_error_class = (!empty($form['form_state']['errors'])) ? 'has-errors' : '';

  $form_html = '<form id="form--' . $form_attr_id . '" class="form form--' . $form_attr_id . ' ' . $form_error_class .'" method="' . $form['form_state']['method'] . '" action="#" enctype="multipart/form-data">';

  if(!empty($form['form_items']) && is_array($form['form_items'])) {
    foreach($form['form_items'] as $key => $form_item) {

      if(empty($form_item['#type']) || (isset($form_item['#access']) && $form_item['#access'] === false))
        continue;

      $form_item['#prefix'] = (!empty($form_item['#prefix'])) ? $form_item['#prefix'] : '';
      $form_item['#suffix'] = (!empty($form_item['#suffix'])) ? $form_item['#suffix'] : '';
      $form_item['#required'] = (!empty($form_item['#required'])) ? $form_item['#required'] : false;
      $form_item['#maxlength'] = (!empty($form_item['#maxlength'])) ? $form_item['#maxlength'] : '';
      $form_item['#multiple'] = (!empty($form_item['#multiple'])) ? $form_item['#multiple'] : false;
      $form_item['#options'] = (!empty($form_item['#options'])) ? $form_item['#options'] : array();
      $form_item['#attributes'] = (!empty($form_item['#attributes'])) ? $form_item['#attributes'] : array();

      // Set empty default value
      if(empty($form_item['#default_value'])) {
        if($form_item['#type'] == 'checkboxes' || (($form_item['#type'] == 'select') && $form_item['#multiple'])) {
          $form_item['#default_value'] = array();
        } else {
          $form_item['#default_value'] = '';
        }
      }

      // Set submitted value as default field value
      if(array_key_exists($key, $form['form_state']['values'])) {
        $form_item['#default_value'] = $form['form_state']['values'][$key];
      }

      $errors = array();

      // Set validation errors
      if(array_key_exists($key, $form['form_state']['errors'])) {
        $errors = $form['form_state']['errors'][$key];
      }

      $form_html .= wp_render_form_item($key, $form_item, $errors);
    }
  }

  $form_html .= '</form>';

  return $form_html;
}

/**
 * wp_render_form_item
 * @param $key
 * @param $form_item
 * @param array $errors
 * @return string
 * @throws Exception
 */
function wp_render_form_item($key, $form_item, $errors = array()) {

  if(empty($form_item['#type']))
    throw new Exception('WP Form : Form item type is not defined.');

  $template_item = __DIR__ . '/templates/form--item--' . $form_item['#type'] . '.tpl.php';

  if(!file_exists($template_item))
    throw new Exception('WP Form : Form item "' . $form_item['#type'] . '" template does not exist.');

  if(!empty($errors)) {
    $template_errors = __DIR__ . '/templates/form--errors.tpl.php';
    $errors = wp_form_render_template($template_errors, array('errors' => $errors));
  } else {
    $errors = '';
  }

  return wp_form_render_template($template_item, array(
    'form_item_key' => $key,
    'form_item' => $form_item,
    'errors' => $errors
  ));
}

/**
 * Render template
 * @param $template_name
 * @param array $variables
 * @return string
 */
function wp_form_render_template($template_name, $variables = array()) {

  ob_start();
  extract($variables);
  include($template_name);
  $content = ob_get_contents();
  ob_end_clean();

  return $content;
}

/**
 * wp form get values
 */
function wp_form_get_values() {

  if(empty($_REQUEST['wp_form_id']))
    return;

  global $wp_form_state;

  // Get form values
  if(function_exists($_REQUEST['wp_form_id'])) {
    foreach($_REQUEST as $key => $value) {
      if(!is_array($value)) {
        $wp_form_state['values'][$key] = filter_var($value, FILTER_SANITIZE_STRING);
      } else {
        $wp_form_state['values'][$key] = $value;
      }
    }

    if(!empty($_FILES)) {
      foreach($_FILES as $key => $file) {
        $wp_form_state['values'][$key] = $file;
        $wp_form_state['values'][$key]['extension'] = pathinfo($file['name'], PATHINFO_EXTENSION);
      }
    }
  }
}

add_action('wp_form_get_values_action', 'wp_form_get_values', 1, 0);
do_action('wp_form_get_values_action');

/**
 * Global form validation callback
 */
function wp_form_validation() {

  if(empty($_REQUEST['wp_form_id']) || (!empty($_REQUEST['wp_form_id']) && !function_exists($_REQUEST['wp_form_id'])))
    return;

  global $wp_form_state;

  $form_id = $_REQUEST['wp_form_id'];
  $form_validation_callback = $form_id . '_validate';
  $form_submit_callback = $form_id . '_submit';
  $form = wp_get_form($form_id, array(), false);
  $form_state['values'] = $form['form_state']['values'];

  if(!wp_verify_nonce($form_state['values'][$form_id . '_nonce'], 'form_nonce')) {
    $wp_form_state['errors']['form_nonce'][] = 'Token invalide';
  }

  // Call form validation callback
  if(function_exists($form_validation_callback))
    $form_validation_callback($form['form_items'], $form_state);

  // Call form submit callback
  if(function_exists($form_submit_callback) && empty($wp_form_state['errors']))
    $form_submit_callback($form['form_items'], $form_state);
}

add_action('wp_form_validation_action', 'wp_form_validation', 2, 0);
do_action( 'wp_form_validation_action');

/**
 * WP Form set error
 * @param $field
 * @param $error
 */
function wp_form_set_error($field, $error) {

  global $wp_form_state;

  $wp_form_state['errors'][$field][] = $error;
}

/**
 * Validate email domain
 * @param $email
 * @return bool
 */
function validateEmailDomain($email) {
  $blacklist = array("yopmail.com", "jetable.org", "link2mail.net", "0-mail.com", "brefmail.com", "tempinbox.com",
    "yopmail.fr", "jetable.fr", "link2mail.fr", "0-mail.fr", "brefmail.fr", "tempinbox.fr", "yopmail.net");

  list($user, $domain) = explode('@', $email);
  $domain = strtolower($domain);

  if (in_array($domain, $blacklist)) {
    return false;
  }

  return true;
}

/**
 * Verify Google Recaptcha
 * @param $code
 * @param null $ip
 * @return bool
 */
function isRecaptchaValid($code, $ip = null) {
  if (empty($code)) {
    return false;
  }

  $secret = get_field('gr_site_secret', 'option');
  $secret = trim($secret);

  $params = [
    'secret' => $secret,
    'response' => $code
  ];
  if ($ip) {
    $params['remoteip'] = $ip;
  }
  $url = "https://www.google.com/recaptcha/api/siteverify?" . http_build_query($params);
  if (function_exists('curl_version')) {
    $curl = curl_init($url);
    curl_setopt($curl, CURLOPT_HEADER, false);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($curl, CURLOPT_TIMEOUT, 1);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
    $response = curl_exec($curl);
  } else {
    // if curl is not available
    $response = file_get_contents($url);
  }

  if (empty($response) || is_null($response)) {
    return false;
  }

  $json = json_decode($response);
  return $json->success;
}

/**
 * Google Recaptcha SiteKey JS
 */
function wp_form_recaptcha_js() {

  $GrSiteKey = get_field('gr_site_key', 'option');

  if(!empty($GrSiteKey)) {
    echo '<script>Google = {GrSiteKey: "' . $GrSiteKey . '" }</script>';
  }
}
add_action('wp_head', 'wp_form_recaptcha_js');


function test_example_form($form, &$form_state) {

  $form['firstname'] = array(
    '#type' => 'textfield',
    '#title' => 'Name',
    '#required' => false,
    '#attributes' => array('placeholder' => 'Name')
  );

  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => 'Description',
    '#required' => true,
  );

  $form['color'] = array(
    '#type' => 'select',
    '#title' => 'Couleur',
    '#options' => array(
      '_none' => 'Choisissez une couleur',
      1 => 'Rouge',
      2 => 'Vert',
      3 => 'Bleu',
    )
  );

  $form['copy'] = array(
    '#type' => 'checkbox',
    '#title' => 'Send me a copy'
  );

  $form['color_choices'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Couleurs',
    '#options' => array(
      1 => 'Rouge',
      2 => 'Vert',
      3 => 'Bleu',
    )
  );

  $form['color_choice'] = array(
    '#type' => 'radios',
    '#title' => 'Couleurs',
    '#options' => array(
      1 => 'Violet',
      2 => 'Noir',
      3 => 'Blanc',
    )
  );

  $form['fichier'] = array(
    '#type' => 'file',
    '#title' => 'Fichier',
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Valider'
  );

  return $form;
}

function test_example_form_validate($form, $form_state) {

  if(empty($form_state['values']['firstname'])) {
    wp_form_set_error('firstname', 'Le nom est obligatoire');
  }
}

function test_example_form_submit($form, $form_state) {
  // Form valid : do something...
}
