"use strict";
require('shelljs/global')
const chalk       = require('chalk')
const logUpdate   = require('log-update')
const configstore = require('configstore')
const timeago     = require('./timeago')

// var ico = require('figures');
// https://github.com/sindresorhus/figures/blob/master/index.js

const gd = {
  package: null,
  context: null,
  current_server: null
}

gd.conf = (force_reload = false) => {

  if(force_reload){
    gd.package = require('./package-finder').get(force_reload)
    return gd.package
  }

  gd.package = gd.package ? gd.package : require('./package-finder').get()
  return gd.package
}

gd.localConf = () => { /* INIT LOCAL (project) CONF */
  const conf = gd.conf();
  try{
    return new configstore(conf.name, { });
  } catch(err) {
    console.log(err.message);
    /*console.log(err.stack);*/
  }
}

gd.globalConf = () => { /* INIT GLOBAL (computer) CONF */
  const conf = gd.conf();
  try{
    return new configstore('gd-cli', {
      db_user: conf.servers.dev.db_user,
      db_pass: conf.servers.dev.db_pass
    });
  } catch(err) {
    console.log(err.message);
    /*console.log(err.stack);*/
  }
}


gd.db = (server) => {

  const conf = gd.conf();
  const conf_local = gd.localConf();

  // get default db_name conf
  let db = conf.config.db_name;

  // get server specific db_name if exist
  if(conf.servers[server].db_name) {
    db = conf.servers[server].db_name;
  }

  // get local conf db_name if specified
  if(server === 'dev' && conf_local.get('db_name')) {
    db = conf_local.get('db_name');
  }

  return db;
};

gd.dev_url = () => {

  const conf = gd.conf();
  const conf_local = gd.localConf();

  // get default dev url
  let url = conf.servers.dev.url;

  // get local conf url if specified
  if(conf_local.get('url')) {
    url = conf_local.get('url');
  }

  return url;
};

gd.server = (server) => {

  if(!server && gd.current_server) {
    return gd.current_server
  }

  const conf = gd.conf()

  const default_server = conf.config['default_server'] || 'dev';

  server = server || default_server;

  if(!conf.servers[server]) {

    gd.error(`Error: Server specified does not exist : ${server}`);
    gd.status(`Servers available are:`);

    for (let server in conf.servers){
      if(server === default_server){
        gd.status(`- ${server} (default)`);
      }else{
        gd.status(`- ${server}`);
      }
    }
    exit()
  }
  else {
    gd.current_server = server
  }

  return server;
};

gd.branch = (server) => {

  const conf = gd.conf()
  const conf_local = gd.localConf();

  let branch = '';

  // get default config branch
  if(conf.config['branch']) {
    branch = conf.config.branch;
  }

  // get server specific branch if exist
  if(conf.servers[server].branch) {
    branch = conf.servers[server].branch;
  }

  // get local conf branch if specified
  if(conf_local.get('branch')) {
    branch = conf_local.get('branch');
  }

  return branch;
};


gd.token = () => {

  const conf = gd.conf()
  const conf_global = gd.globalConf();

  if(conf_global.get('token')) {
    return conf_global.get('token')
  }
  const token = conf.repository.token

  return token;
}


gd.username = () => {

  const conf = gd.conf()
  const conf_global = gd.globalConf();

  if(conf_global.get('username')) {
    return conf_global.get('username')
  }
  const usernameHome = require('path').basename(require('user-home'))

  return usernameHome;
}

gd.isGui = () => {
  return gd.context === 'gui'
}


gd.getArg = (args) => {

  return args[0] && !require('is-object')(args[0]) ? args[0] : false
}


gd.log = (log) => {
  console.log(chalk.dim.green(`> ${log} \n`));
};
gd.status = (status) => {
  console.log(chalk.italic.yellow(`${status} \n`));
};
gd.error = (error) => {
  console.log(chalk.dim.yellow(`${error} \n`));
};
gd.exit = (error) => {
  gd.error(`${error} \n`);
  exit();
};
gd.dbg = (log, _exit) => {
  console.log(`
===============DEBUG===============

${log}

=============END-DEBUG=============
`);

  if(_exit) exit();
};


gd.exec = (cmd, cb, message) => {

  exec(cmd, function(code, stdout, stderr) {

    logUpdate.clear();
    // logUpdate.done();

    if(stderr
    && stderr.indexOf('commit your changes') < 0
    && stderr.indexOf('Using a password on the command line') < 0
    && stderr.indexOf('the graceful-fs module') < 0
    && stderr.indexOf('cache was cleared in') < 0
    && stderr.indexOf('.htaccess: Permission denied') < 0
    && stderr.indexOf('The remote end hung up unexpectedly') < 0
    && stderr.indexOf('-> FETCH_HEAD') < 0) {
      gd.error(stderr);

      if(stderr.indexOf('mysqldump: Got error: ') > -1) {
        gd.exit(`Can't get a dump, check db name in package.json`)
      }
    }
    if(stdout) {
      console.log('\n');
    }

    gd.throbber.stop();

    if(message) {
      gd.log(message);
    }
    if(cb) {
      cb();
    }
    else {
      gd.autoClose();
    }

  });
};

gd.execraw = (cmd, cb, message) => {

  exec(cmd, function(code, stdout, stderr) {

    if(message) {
      gd.log(message);
    }
    if(cb) {
      cb();
    }
    else {
      gd.autoClose();
    }

  });
};


gd.autoClose = () => {

  if(!gd.isGui()) return;

  var i = 18 * 1
  setInterval(function(){

    logUpdate.clear();
    logUpdate(`This window will be closed in \u001b[33m${i--}\u001b[0m (ctrl+c to cancel)`);

    if(!i) exit();

  }, 1000)
}


gd.throbber = {
  running: null,
  from: null,
};

gd.throbber.stop = () => {
  logUpdate.clear();
  clearInterval(gd.throbber.running);
  gd.timer = 0;
};

gd.throbber.start = (status = 'Loading') => {

  const ta = timeago();
  gd.throbber.from = new Date();

  let i = 0;
  let j = 0;
  const frames = ['-', '\\', '|', '/'];
  const dots = ['', '.', '..', '...', '.....', '.....', '....', '...', '..', '.'];

  let interval_ms = 100;

  gd.throbber.running = setInterval(() => {

    gd.throbber.timer += interval_ms / 1000;

    const frame = frames[j = ++j % frames.length];
    const dot   = dots  [i = ++i % dots.length];

    logUpdate(
`
   ${frame} ${status}${dot}

   Time elapsed: \u001b[33m${ta.ago(gd.throbber.from,'')}\u001b[0m
`
    );
  }, interval_ms);
};


module.exports = gd
